@@ -6,7 +6,9 @@
 #include "FormatArray.h"
 #include "APETag.h"
 #include "FLACTag.h"
-#include "IO.h"
+#include "IAPEIO.h"
+#include "WAVInputSource.h"
+#include "ID3v2Tag.h"
 
 using namespace APE;
 
@@ -22,7 +24,7 @@ CMACProcessFiles::~CMACProcessFiles()
 {
     // stop
     for (int z = 0; z < m_paryFiles->GetSize(); z++)
-        m_paryFiles->ElementAt(z).m_nKillFlag = KILL_FLAG_STOP;
+        m_paryFiles->ElementAt(z)->m_nKillFlag = KILL_FLAG_STOP;
 
     // wait until we're done processing
     bool bProcessing = true;
@@ -31,7 +33,7 @@ CMACProcessFiles::~CMACProcessFiles()
         bProcessing = false;
         for (int z = 0; z < m_paryFiles->GetSize(); z++)
         {
-            if (m_paryFiles->ElementAt(z).m_bProcessing)
+            if (m_paryFiles->ElementAt(z)->m_bProcessing)
                 bProcessing = true;
         }
 
@@ -63,7 +65,7 @@ bool CMACProcessFiles::Pause(bool bPause)
     // update file status
     for (int z = 0; z < m_paryFiles->GetSize(); z++)
     {
-        m_paryFiles->ElementAt(z).m_nKillFlag = bPause ? KILL_FLAG_PAUSE : KILL_FLAG_CONTINUE;
+        m_paryFiles->ElementAt(z)->m_nKillFlag = bPause ? KILL_FLAG_PAUSE : KILL_FLAG_CONTINUE;
     }
 
     // update the pause timer
@@ -94,7 +96,7 @@ bool CMACProcessFiles::Stop(bool bImmediately)
     if (bImmediately)
     {
         for (int z = 0; z < m_paryFiles->GetSize(); z++)
-            m_paryFiles->ElementAt(z).m_nKillFlag = KILL_FLAG_STOP;
+            m_paryFiles->ElementAt(z)->m_nKillFlag = KILL_FLAG_STOP;
     }
 
     // flag the stop (it will happen eventually if we didn't do an immediate stop)
@@ -106,11 +108,11 @@ bool CMACProcessFiles::Stop(bool bImmediately)
 bool CMACProcessFiles::ProcessFile(int nIndex)
 {
     // setup
-    m_paryFiles->ElementAt(nIndex).m_bStarted = true;
-    TICK_COUNT_READ(m_paryFiles->ElementAt(nIndex).m_dwStartTickCount);
+    m_paryFiles->ElementAt(nIndex)->m_bStarted = true;
+    TICK_COUNT_READ(m_paryFiles->ElementAt(nIndex)->m_dwStartTickCount);
 
     // create the thread
-    std::thread Thread(ProcessFileThread, &m_paryFiles->ElementAt(nIndex));
+    std::thread Thread(ProcessFileThread, m_paryFiles->ElementAt(nIndex));
 
     // spin off the thread
     Thread.detach();
@@ -375,7 +377,7 @@ void CMACProcessFiles::ProcessFileThread(MAC_FILE * pInfo)
                     {
                         int nBufferBytes = static_cast<int>(spFLACTag->m_nPictureBytes) + spFLACTag->m_strImageExtension.GetLength() + 1;
                         CSmartPtr<char> spBuffer(new char [static_cast<size_t>(nBufferBytes)], true);
-                        CSmartPtr<char> spExtensionANSI(CAPECharacterHelper::GetANSIFromUTF16(spFLACTag->m_strImageExtension), true);
+                        CSmartPtr<char> spExtensionANSI(CAPECharacterHelper::GetANSIFromUTFN(spFLACTag->m_strImageExtension), true);
                         memcpy(spBuffer, spExtensionANSI, static_cast<size_t>(spFLACTag->m_strImageExtension.GetLength())); // extension
                         spBuffer[spFLACTag->m_strImageExtension.GetLength()] = 0; // NULL after the extension
                         memcpy(&spBuffer[spFLACTag->m_strImageExtension.GetLength() + 1], spFLACTag->m_spPicture, spFLACTag->m_nPictureBytes); // image data
@@ -385,6 +387,53 @@ void CMACProcessFiles::ProcessFileThread(MAC_FILE * pInfo)
                     spAPETagNew->Save(false);
                 }
             }
+            else if ((nRetVal == ERROR_SUCCESS) && (pInfo->GetOutputExtension() == _T(".ape")) && (pInfo->m_Tag.m_bFoundTag != false))
+            {
+                // copy tags from WAV, AIFF, etc to APE files
+                CSmartPtr<APE::IAPETag> spAPETagNew(new CAPETag(pInfo->m_strOutputFilename, true));
+
+                if (pInfo->m_Tag.m_spTitle != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_TITLE, pInfo->m_Tag.m_spTitle);
+                if (pInfo->m_Tag.m_spArtist != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_ARTIST, pInfo->m_Tag.m_spArtist);
+                if (pInfo->m_Tag.m_spAlbumArtist != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_ALBUM_ARTIST, pInfo->m_Tag.m_spAlbumArtist);
+                if (pInfo->m_Tag.m_spAlbum != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_ALBUM, pInfo->m_Tag.m_spAlbum);
+                if (pInfo->m_Tag.m_spYear != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_YEAR, pInfo->m_Tag.m_spYear);
+                if (pInfo->m_Tag.m_spGenre != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_GENRE, pInfo->m_Tag.m_spGenre);
+                if (pInfo->m_Tag.m_spComment != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_COMMENT, pInfo->m_Tag.m_spComment);
+                if (pInfo->m_Tag.m_spComposer != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_COMPOSER, pInfo->m_Tag.m_spComposer);
+                if (pInfo->m_Tag.m_spTrackNumber != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_TRACK, pInfo->m_Tag.m_spTrackNumber);
+                if (pInfo->m_Tag.m_spDiscNumber != APE_NULL)
+                    spAPETagNew->SetFieldString(APE_TAG_FIELD_DISC, pInfo->m_Tag.m_spDiscNumber);
+                if (pInfo->m_Tag.m_spImage != NULL)
+                {
+                    // get the MIME type
+                    CString strMimeType; strMimeType.SetString(pInfo->m_Tag.m_spImageMimeType);
+
+                    // trim if it's like image/JPEG
+                    if (strMimeType.Find(_T("/")))
+                        strMimeType = strMimeType.Right(strMimeType.GetLength() - strMimeType.Find(_T("/")) - 1);
+
+                    // build the image tag
+                    CSmartPtr<char> spExtensionANSI(CAPECharacterHelper::GetANSIFromUTFN(strMimeType), true);
+                    int nExtensionLength = static_cast<int>(strlen(spExtensionANSI));
+                    int nBufferBytes = static_cast<int>(pInfo->m_Tag.m_nImageBytes) + static_cast<int>(nExtensionLength) + 1;
+                    CSmartPtr<char> spBuffer(new char [static_cast<size_t>(nBufferBytes)], true);
+                    memcpy(spBuffer, spExtensionANSI, static_cast<size_t>(nExtensionLength)); // extension
+                    spBuffer[nExtensionLength] = 0; // NULL after the extension
+                    memcpy(&spBuffer[nExtensionLength + 1], pInfo->m_Tag.m_spImage.GetPtr(), static_cast<size_t>(pInfo->m_Tag.m_nImageBytes)); // image data
+                    spAPETagNew->SetFieldBinary(APE_TAG_FIELD_COVER_ART_FRONT, spBuffer, static_cast<intn>(nBufferBytes), TAG_FIELD_FLAG_DATA_TYPE_BINARY);
+                }
+
+                spAPETagNew->Save(false);
+            }
 
             // verify (if necessary)
             if (bVerifyWhenDone && (nRetVal == ERROR_SUCCESS))
