@@ -5,8 +5,7 @@
 #include "BufferIO.h"
 #include "MACLib.h"
 #include "GlobalFunctions.h"
-#include "FloatTransform.h"
-#include "CharacterHelper.h"
+#include "WholeFileIO.h"
 
 namespace APE
 {
@@ -14,7 +13,7 @@ namespace APE
 /**************************************************************************************************
 Input source creation
 **************************************************************************************************/
-CInputSource * CInputSource::CreateInputSource(const str_utfn * pSourceName, WAVEFORMATEX * pwfeSource, int64 * pTotalBlocks, int64 * pHeaderBytes, int64 * pTerminatingBytes, int32 * pFlags, IID3v2Tag * pTag, int * pErrorCode)
+CInputSource * CInputSource::CreateInputSource(const str_utfn * pSourceName, WAVEFORMATEX * pwfeSource, int64 * pTotalBlocks, int64 * pHeaderBytes, int64 * pTerminatingBytes, int32 * pFlags, IID3v2Tag * pTag, bool bReadFullInputForUnknownLength, int * pErrorCode)
 {
     // error check the parameters
     if ((pSourceName == APE_NULL) || (wcslen(pSourceName) == 0))
@@ -55,7 +54,7 @@ CInputSource * CInputSource::CreateInputSource(const str_utfn * pSourceName, WAV
     if (CWAVInputSource::GetHeaderMatches(aryHeader))
     {
         if (pErrorCode) *pErrorCode = ERROR_SUCCESS;
-        CInputSource * pWAV = new CWAVInputSource(spIO, pwfeSource, pTotalBlocks, pHeaderBytes, pTerminatingBytes, pTag, pErrorCode);
+        CInputSource * pWAV = new CWAVInputSource(spIO, pwfeSource, pTotalBlocks, pHeaderBytes, pTerminatingBytes, pTag, bReadFullInputForUnknownLength, pErrorCode);
         spIO.SetDelete(false);
         if (pWAV->GetFloat())
             *pFlags |= APE_FORMAT_FLAG_FLOATING_POINT;
@@ -69,7 +68,7 @@ CInputSource * CInputSource::CreateInputSource(const str_utfn * pSourceName, WAV
         spIO.SetDelete(false);
         if (pAIFF->GetIsBigEndian())
             *pFlags |= APE_FORMAT_FLAG_BIG_ENDIAN;
-        if (pwfeSource->wBitsPerSample == 8)
+        if ((pwfeSource->wBitsPerSample == 8) && pAIFF->m_bSigned8Bit)
             *pFlags |= APE_FORMAT_FLAG_SIGNED_8_BIT;
         if (pwfeSource->wFormatTag == WAVE_FORMAT_IEEE_FLOAT)
             *pFlags |= APE_FORMAT_FLAG_FLOATING_POINT;
@@ -227,7 +226,7 @@ struct CWAVInputSource::WAV_FORMAT_HEADER
     return false;
 }
 
-CWAVInputSource::CWAVInputSource(IAPEIO * pIO, WAVEFORMATEX * pwfeSource, int64 * pTotalBlocks, int64 * pHeaderBytes, int64 * pTerminatingBytes, IID3v2Tag * pTag, int * pErrorCode)
+CWAVInputSource::CWAVInputSource(IAPEIO * pIO, WAVEFORMATEX * pwfeSource, int64 * pTotalBlocks, int64 * pHeaderBytes, int64 * pTerminatingBytes, IID3v2Tag * pTag, bool bReadFullInputForUnknownLength, int * pErrorCode)
 {
     m_bIsValid = false;
     m_nDataBytes = 0;
@@ -247,6 +246,18 @@ CWAVInputSource::CWAVInputSource(IAPEIO * pIO, WAVEFORMATEX * pwfeSource, int64
     // store the reader
     m_spIO.Assign(pIO);
 
+    // if we're an undefined size, try reading the whole file
+    if (bReadFullInputForUnknownLength && (pIO->GetSize() == APE_FILE_SIZE_UNDEFINED))
+    {
+        CWholeFileIO * pWholeFile = CreateWholeFileIO(m_spIO, APE_FILE_SIZE_UNDEFINED);
+        if (pWholeFile != APE_NULL)
+        {
+            m_spIO.SetDelete(false);
+            m_spIO.Assign(pWholeFile);
+            m_spIO.SetDelete(true);
+        }
+    }
+
     // read to a buffer so pipes work (that way we don't have to seek back to get the header)
     m_spIO.SetDelete(false);
     m_spIO.Assign(new CBufferIO(m_spIO, APE_BYTES_IN_KILOBYTE * 256));
@@ -654,6 +665,7 @@ CAIFFInputSource::CAIFFInputSource(IAPEIO * pIO, WAVEFORMATEX * pwfeSource, int6
     m_nTerminatingBytes = 0;
     m_bLittleEndian = false;
     m_bFloat = false;
+    m_bSigned8Bit = true;
     m_pTag = pTag;
 
     if (pIO == APE_NULL || pwfeSource == APE_NULL)
@@ -794,10 +806,19 @@ int CAIFFInputSource::AnalyzeSource()
                     {
                         // this means we're a supported file
                     }
+                    else if (memcmp(spBuffer, "twos", 4) == 0)
+                    {
+                        // this is the reverse of sowt so the endian is flipped
+                        m_bLittleEndian = false;
+                    }
                     else if (memcmp(spBuffer, "sowt", 4) == 0)
                     {
                         m_bLittleEndian = true;
                     }
+                    else if (memcmp(spBuffer, "raw ", 4) == 0)
+                    {
+                        m_bSigned8Bit = false;
+                    }
                     else if ((memcmp(spBuffer, "fl32", 4) == 0) || (memcmp(spBuffer, "FL32", 4) == 0))
                     {
                         m_bFloat = true;
@@ -893,7 +914,10 @@ int CAIFFInputSource::GetData(unsigned char * pBuffer, int nBlocks, int * pBlock
         return ERROR_IO_READ;
 
     if (m_wfeSource.wBitsPerSample == 8)
-        Convert8BitSignedToUnsigned(pBuffer, m_wfeSource.nChannels, nBlocks);
+    {
+        if (m_bSigned8Bit)
+            Convert8BitSignedToUnsigned(pBuffer, m_wfeSource.nChannels, nBlocks);
+    }
 #if APE_BYTE_ORDER == APE_LITTLE_ENDIAN
     else if (!m_bLittleEndian)
         SwitchBufferBytes(pBuffer, m_wfeSource.wBitsPerSample / 8, nBlocks * m_wfeSource.nChannels);
